@@ -1,4 +1,4 @@
-// Copyright 2020 The Grin Developers
+// Copyright 2021 The Grin Developers
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -25,8 +25,6 @@ use chrono::prelude::*;
 use serde::de::{SeqAccess, Visitor};
 use serde::{Deserialize, Deserializer};
 
-use grin_store;
-
 use crate::chain;
 use crate::chain::txhashset::BitmapChunk;
 use crate::core::core;
@@ -391,6 +389,8 @@ bitflags! {
 		const TX_KERNEL_HASH = 0b0000_1000;
 		/// Can provide PIBD segments during initial byte download (fast sync).
 		const PIBD_HIST = 0b0001_0000;
+		/// Can provide historical blocks for archival sync.
+		const BLOCK_HIST = 0b0010_0000;
 	}
 }
 
